﻿using Kivii.Domain.Lac.Lims.Transforms;
using Kivii;
using Kivii.Lims.Entities;
using Kivii.Lac.Lims.Extensions;
using Kivii.Linq;
using Kivii.Products.Entities;
using Kivii.Standards.Entities;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text.RegularExpressions;

namespace Kivii.Domain.Lac.Lims.Extensions
{
    public static class ReportExtension
    {

        #region 生成报告结论方法
        public static void GenerateConclusion(this Report report, string first, string second, string third, IDbConnection conn = null)
        {
            report.ThrowIfNull("参数异常");
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == report.Kvid).OrderBy(o => o.SortId).ThenBy(o => o.SortIdEx);

            //整个报告的全部项目
            var allReportItems = conn.Select(queryReportItems);

            //全项中的大项
            var primaryAllReportItems = allReportItems.filterReportItem();
            if (primaryAllReportItems.IsNullOrEmpty()) throw new Exception("该报告无项目！");

            if(report.TestJudgement == null||report.TestJudgement == string.Empty) throw new Exception("报告无判定结果，无法自动生成报告结论！");
            //根据报告的合格与不合格，筛选对应的项目，包好实测值项目
            primaryAllReportItems = primaryAllReportItems.Where(o => o.Judgement == report.TestJudgement || o.Judgement == null || o.Judgement == string.Empty || o.Judgement== "—" || o.Judgement == "-").ToList();
            var testStandards = string.Empty;//检验依据
            var testDetections = string.Empty;//检验项目
            var measureDetections = string.Empty;//实测值项目
            var testStandardsList = new List<string>();
            var testDetectionsList = new List<string>();
            var measureDetectionList = new List<string>();

            #region 组合实测值项目
            var measureReportItems = primaryAllReportItems.Where(o => o.Judgement == null || o.Judgement == string.Empty || o.Judgement == "—" || o.Judgement == "-").ToList();
            foreach(var measureItem in measureReportItems)
            {
                measureItem.Title = measureItem.Title.Replace("(公定)", "").Replace("（公定）", "");
                measureDetectionList.Add(measureItem.Title);
            }
            measureDetectionList = measureDetectionList.Distinct().ToList();
            measureDetections = String.Join("、", measureDetectionList);
            #endregion

            #region 组合检验依据
            var groupStandards = primaryAllReportItems.GroupBy(o => new { o.StandardCode, o.StandardTitle });

            var countSafety = groupStandards.Count(o => o.Key.StandardCode.Contains("18401")) + groupStandards.Count(o => o.Key.StandardCode.Contains("31701")) + groupStandards.Count(o => o.Key.StandardCode.Contains("20400"));
            var countStandards = groupStandards.Count();
            //针对只有国家强制安全标准的项目，生成结论的格式单独处理
            if (countSafety == countStandards)
            {
                foreach (var kv in groupStandards)
                {
                    testStandardsList.Add(kv.Key.StandardCode + (kv.Key.StandardTitle.IsNullOrEmpty() ? "" : "《" + kv.Key.StandardTitle + "》"));
                }
            }
            else
            {
                foreach (var kv in groupStandards)
                {
                    if (kv.Key.StandardCode.Contains("18401") || kv.Key.StandardCode.Contains("31701") || kv.Key.StandardCode.Contains("20400"))
                    {
                        testStandardsList.Add(kv.Key.StandardCode + (kv.Key.StandardTitle.IsNullOrEmpty() ? "" : "《" + kv.Key.StandardTitle + "》") + " " + report.SampleGradeSafety);
                    }
                    else
                    {
                        testStandardsList.Add(kv.Key.StandardCode + (kv.Key.StandardTitle.IsNullOrEmpty() ? "" : "《" + kv.Key.StandardTitle + "》"));

                    }
                }
            }
            
            testStandardsList = testStandardsList.Distinct().ToList();
            testStandards = String.Join("、", testStandardsList);

            #endregion

            var conclusion = string.Empty;
            //2021-10-10升级，若委托报告出现项目不合格，且不合格项目检验依据不在报告的检验依据中，全部返回不符合
            //2021-10-11升级，若委托报告出现项目合格，且合格项目检验依据不在报告的检验依据中，全部返回不符合
            foreach (var testStandard in testStandardsList)
            {
                //截取最后两个字符，避免出现A类等
                //Console.WriteLine("-------------------" + testStandard.Substring(0, testStandard.Length - 3));
                //Console.WriteLine("-------------------" + report.TestStandards);
                //Console.WriteLine("-------------------" + report.TestStandards.Contains(testStandard.Substring(0, testStandard.Length - 3)));
                if (report.TestStandards.Contains(testStandard.Substring(0, testStandard.Length - 3))) continue;
                //如果项目的产品标准不在检验依据中，处理报告的检验依据，加安全等级
                var reportTestStandardLists = report.TestStandards.Split('、');
                var temList = new List<string>();
                foreach(var st in reportTestStandardLists)
                {
                    if(st.Contains("18401")||st.Contains("31701")|st.Contains("20400"))
                    {
                        temList.Add(st + " " + report.SampleGradeSafety);
                    }
                    else
                    {
                        temList.Add(st);
                    }
                }

                testStandards = String.Join("、", temList);
            }
            if (report.TestJudgement == "合格")
            {
                if (countSafety == countStandards) conclusion = $"{first}，所检项目符合{testStandards}标准规定的{report.SampleGradeSafety}要求";
                else conclusion = $"{first}，所检项目符合{testStandards}标准规定的{report.SampleGrade}要求";
                if (measureDetections != string.Empty) conclusion += $"(其中{measureDetections}为实测值)";
                if (third != string.Empty) conclusion = conclusion + "，" + third + "。";
                else conclusion += "。";
            }
            else if (report.TestJudgement == "不合格")
            {
                
                //非监督报告，不需要具体不合格项目
                if (second == string.Empty)
                {
                    if (countSafety == countStandards) conclusion = $"{first}，不符合{testStandards}标准规定的{report.SampleGradeSafety}要求";
                    else conclusion = $"{first}，不符合{testStandards}标准规定的{report.SampleGrade}要求";
                    //2021-10-29 依据蒋洁蓉分管院长的《新系统编制问题》要求
                    //if (measureDetections != string.Empty) conclusion += $"(其中{measureDetections}为实测值)";
                    if (third != string.Empty) conclusion = conclusion + "，" + third + "。";
                    else conclusion += "。";
                }
                else
                {
                    foreach (var en in primaryAllReportItems)
                    {
                        var testStandard = en.StandardCode + (en.StandardTitle.IsNullOrEmpty() ? "" : "《" + en.StandardTitle + "》");
                        if (en.StandardCode.Contains("18401") || en.StandardCode.Contains("31701") || en.StandardCode.Contains("20400"))
                        {
                            testStandard = testStandard + " " + report.SampleGradeSafety;
                        }
                        if (en.ParentKvid != Guid.Empty)
                        {
                            var parentReportItem = allReportItems.Where(o => o.Kvid == en.ParentKvid && o.Type == ReportItemType.SamplePart).FirstOrDefault();
                            if (parentReportItem == null)
                            {
                                testDetectionsList.Add($"{en.Title}{second}不符合{testStandard}标准规定的{report.SampleGrade}");
                            }
                            else
                            {
                                testDetectionsList.Add($"{en.Title}({parentReportItem.Title}){second}不符合{testStandard}标准规定的{report.SampleGrade}要求");
                            }
                        }
                        else
                        {
                            testDetectionsList.Add($"{en.Title}{second}不符合{testStandard}标准规定的{report.SampleGrade}要求");
                        }
                    }
                    testDetectionsList = testDetectionsList.Distinct().ToList();
                    testDetections = String.Join("、", testDetectionsList);
                    conclusion = $"{first}，{testDetections}";
                    //if (measureDetections != string.Empty) conclusion += $"(其中{measureDetections}为实测值)";
                    if (third != string.Empty) conclusion = conclusion + "，" + third + "。";
                    else conclusion += "。";
                }
            }
            else
            {
                throw new Exception("报告无判定结果，无法自动生成报告结论！");
            }
            report.Conclusion = conclusion;
            report.AddOnlyProperties(o => o.Conclusion);
            conn.UpdateOnly(report);
        }
        #endregion

        #region 拷贝粘贴报告涉及的方法
        /// <summary>
        /// 根据拷贝报告重新组织待粘贴报告
        /// 注意:此方法只进行报告数据的重新组合,不涉及数据库更新
        /// </summary>
        /// <param name="report"></param>
        /// <returns></returns>
        public static Report GetPasteReport(this Report report)
        {
            var pasteReport = new Report();
            pasteReport.PopulateWith(report);
            pasteReport.Metadata = new Dictionary<string, string>();
            foreach(var meta in report.Metadata)
            {
                pasteReport.Metadata.Add(meta.Key,meta.Value);
            }
            //重置biz
            pasteReport.BizId = string.Empty;
            pasteReport.BizKvid = Guid.Empty;
            pasteReport.BizType = string.Empty;
            pasteReport.ReportId = string.Empty;
            pasteReport.ReportIdNumber = 0;
            pasteReport.SampleKvid = Guid.Empty;
            pasteReport.SampleSerialNumber = String.Empty;
            pasteReport.SampleSourceKvid = Guid.Empty;
            pasteReport.SampleSourceBatchNumber = String.Empty;
            pasteReport.TestStandards = String.Empty;
            pasteReport.TestDetections = String.Empty;
            pasteReport.TestJudgement = string.Empty;
            //加急系数恢复默认
            pasteReport.UrgentRate = 1;
            //报告结论置空
            pasteReport.Conclusion = string.Empty;
            pasteReport.DealDate = DateTime.Now;
            pasteReport.DeadDate = null;
            pasteReport.ContractReviewDate = null;
            pasteReport.TaskAssignDate = null;
            pasteReport.TestStartDate = null;
            pasteReport.TestFinishDate = null;
            pasteReport.TestFinishDate = null;
            pasteReport.PreparationDate = null;
            pasteReport.ReviewDate = null;
            pasteReport.IssueDate = null;
            pasteReport.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            pasteReport.OperatorName = KiviiContext.CurrentMember.FullName;
            pasteReport.ManagerKvid = Guid.Empty;
            pasteReport.ManagerName = String.Empty;
            pasteReport.PreparerKvid = Guid.Empty;
            pasteReport.PreparerName = String.Empty;
            pasteReport.ReviewerKvid = Guid.Empty;
            pasteReport.ReviewerName = String.Empty;
            pasteReport.IssuerKvid = Guid.Empty;
            pasteReport.IssuerName = String.Empty;
            pasteReport.AmountTest = 0;
            pasteReport.AmountTestPlan = 0;
            pasteReport.AmountSettlement = 0;
            pasteReport.AmountPayment = 0;
            pasteReport.StatusType = ReportStatusType.CommissionAccept;
            pasteReport.Status = (int)ReportStatusType.CommissionAccept;
            pasteReport.Kvid = Guid.NewGuid();
            pasteReport.Metadata["SampleEffectiveDate"] = DateTime.Now.ToString();
            pasteReport.Metadata["OriginalReportId"] = report.ReportId;
            if (report.Metadata.ContainsKey("CallBackFrom")) pasteReport.Metadata["CallBackFrom"] = null;
            if (report.Metadata.ContainsKey("CallBackTo")) pasteReport.Metadata["CallBackTo"] = null;
            if (report.Metadata.ContainsKey("CallBackFromGroups")) pasteReport.Metadata["CallBackFromGroups"] = null;
            if (report.Metadata.ContainsKey("IsCallbackAfterFinish")) pasteReport.Metadata["IsCallbackAfterFinish"] = null;
            if (report.Metadata.ContainsKey("IsDirectReturn")) pasteReport.Metadata["IsDirectReturn"] = null;
            if (report.Metadata.ContainsKey("DirectReturnStatus")) pasteReport.Metadata["DirectReturnStatus"] = null;
            if (report.Metadata.ContainsKey("ReviewingGroup")) pasteReport.Metadata["ReviewingGroup"] = null;
            if (report.Metadata.ContainsKey("ReportRequire")) pasteReport.Metadata["ReportRequire"] = null;
            //清除系统老数据的flowcallback
            if (report.Metadata.ContainsKey("FlowCallBack")) pasteReport.Metadata["FlowCallBack"] = null;
            //报告要求置空
            //标记含量复核为空
            if (report.Metadata.ContainsKey("IsFiberContentCheck")) pasteReport.Metadata["IsFiberContentCheck"] = null;
            if (report.Metadata.ContainsKey("FiberContentCheckName")) pasteReport.Metadata["FiberContentCheckName"] = null;
            if (report.Metadata.ContainsKey("FiberContentCheckTime")) pasteReport.Metadata["FiberContentCheckTime"] = null;
            //标记报告复核为空
            if (report.Metadata.ContainsKey("IsCheck")) pasteReport.Metadata["IsCheck"] = null;
            if (report.Metadata.ContainsKey("CheckName")) pasteReport.Metadata["CheckName"] = null;
            if (report.Metadata.ContainsKey("CheckTime")) pasteReport.Metadata["CheckTime"] = null;
            //三方物料编码置空
            if (report.Metadata.ContainsKey("SampleCode")) pasteReport.Metadata["SampleCode"] = null;
            //客户要求、样品描述置空
            if (report.Metadata.ContainsKey("SampleDescriptionEx")) pasteReport.Metadata["SampleDescriptionEx"] = null;
            if (report.Metadata.ContainsKey("CustomerRequire")) pasteReport.Metadata["CustomerRequire"] = null;
            if (report.Metadata.ContainsKey("testStandardsMethod")) pasteReport.Metadata["testStandardsMethod"] = null;
            //清除报告更改人信息
            if (report.Metadata.ContainsKey("HandlingPersonName")) pasteReport.Metadata["HandlingPersonName"] = null;
            if (report.Metadata.ContainsKey("ResponsibleName")) pasteReport.Metadata["ResponsibleName"] = null;
            //20211213 经讨论,监督报告抽样日期置空,检查封样人员为当前登录人
            if (report.TypeEx == "J")
            {
                pasteReport.Metadata["SamplingDate"] = null;
                pasteReport.Metadata["CheckMemberName"] = KiviiContext.CurrentMember.FullName;
                pasteReport.SampleSourceBatchNumber = report.SampleSourceBatchNumber;
            }
            return pasteReport;
        }
        /// <summary>
        /// 拷贝粘贴项目
        /// </summary>
        /// <param name="reportItemOriginal"></param>
        /// <param name="detections"></param>
        /// <returns></returns>
        public static ReportItem GetPasteReportItem(this ReportItem reportItemOriginal,List<Detection> detections)
        {
            var reportItem = new ReportItem();
            reportItem.PopulateWith(reportItemOriginal);
            //reportItem.Metadata = new Dictionary<string, string>();
            //foreach (var meta in reportItem.Metadata)
            //{
            //    reportItem.Metadata.Add(meta.Key, meta.Value);
            //}
            if (reportItem.DetectionKvid != Guid.Empty) {
                //拷贝时,保留检验项目标题原数据
                if (!detections.IsNullOrEmpty()) {
                    var detection = detections.FirstOrDefault(o => o.Kvid == reportItem.DetectionKvid);
                    if(detection != null) reportItem.Title = detection.Title;
                }
            }
            //拷贝粘贴不粘贴条件值
            if (reportItem.Metadata != null && reportItem.Metadata.ContainsKey("DetectionValueCondition")) reportItem.Metadata["DetectionValueCondition"] = string.Empty;
            //拷贝粘贴不粘贴(标准值更新标记)
            if (reportItem.Metadata != null && reportItem.Metadata.ContainsKey("IsUpdateDetectionValueQualified")) reportItem.Metadata["IsUpdateDetectionValueQualified"] = string.Empty;
            reportItem.TestKvid = Guid.Empty;
            reportItem.TestRootKvid = Guid.Empty;
            reportItem.TestValue = String.Empty;
            reportItem.TestCreatorKvid = Guid.Empty;
            reportItem.TestCreatorName = String.Empty;
            reportItem.TestReviewerKvid = Guid.Empty;
            reportItem.TestReviewerName = String.Empty;
            reportItem.TestCreateTime = null;
            reportItem.TestReviewTime = null;
            reportItem.TestOriginalFilePath = String.Empty;
            reportItem.Judgement = String.Empty;
            reportItem.StatusType = ReportItemStatusType.BeforeTest;
            reportItem.Status = (int)ReportItemStatusType.BeforeTest;
            return reportItem;
        }
        /// <summary>
        /// 拷贝粘贴项目(升级)
        /// </summary>
        /// <param name="reportItemOriginal">拷贝父本(检验项目)</param>
        /// <param name="report">拷贝父本(报告)获取标准值时需要用到样品等级等属性</param>
        /// <param name="detection">检验项目</param>
        /// <param name="detectionValues">标准值集合</param>
        /// <returns></returns>
        public static ReportItem GetPasteReportItemUpgrade(this ReportItem reportItemOriginal, Report report,Detection detection,List<DetectionValue> detectionValues,List<Standard> standards,bool isCopyDeteValueCustomerRequired=false, List<Price> prices = null)
        {
            var reportItem = new ReportItem();
            reportItem.PopulateWith(reportItemOriginal);
            if (detection != null)
            {
                //必填项,无需判断是否为空
                reportItem.Title = detection.Title;
                reportItem.DetectionTitle= detection.Title;
                //新创建的项目的检测方法要重新去标准里计算 2022-08-16
                if (!detection.CodeActual.IsNullOrEmpty()) reportItem.DetectionCodeActual = (detection.CodeActual.Split(';').Length > 1) ? detection.CodeActual.Split(';')[0] : detection.CodeActual;
                else reportItem.DetectionCodeActual = string.Empty;
                if (reportItem.Metadata.ContainsKey("DetectionCodeActual")) reportItem.Metadata["DetectionCodeActual"] = detection.CodeActual;
                else reportItem.Metadata.Add("DetectionCodeActual", detection.CodeActual);
                //新创建的项目的检测方法说明要重新去标准里计算 2023-03-07
                if (!detection.MethodDescription.IsNullOrEmpty())reportItem.DetectionCodeActual = reportItem.DetectionCodeActual + " " + detection.MethodDescription;
                if (reportItem.Metadata.ContainsKey("DetectionMethodDescription")) reportItem.Metadata["DetectionMethodDescription"] = detection.MethodDescription;
                else reportItem.Metadata.Add("DetectionMethodDescription", detection.MethodDescription);
                //新创建的项目的项目说明要重新去标准里计算 2023-01-29
                if (reportItem.Metadata.ContainsKey("DetectionDescription")) reportItem.Metadata["DetectionDescription"] = detection.Description;
                else reportItem.Metadata.Add("DetectionDescription", detection.Description);
                //新创建的项目的数值单位要重新去标准里计算 2024-03-25
                reportItem.DetectionUnit = detection.Unit;
                //获取标准值
                var detectionValueQualified = detection.GetQualifiedValueUpgrade(report.SampleGrade, report.SampleGradeSafety, report.SampleGradeFunction, detectionValues);
                reportItem.DetectionValueQualified = detectionValueQualified;
                //更新价格类型为默认价格类型，不能用原数据的价格类型
                if (!prices.IsNullOrEmpty())
                {
                    var currentPrices = prices.Where(o => o.GoodsKvid == detection.GoodsKvid).OrderByDescending(o => o.IsDefault).ToList();
                    if (!currentPrices.IsNullOrEmpty())
                    {
                        Price price = null;
                        if (currentPrices.Count == 1)
                        {
                            price = currentPrices[0];
                            reportItem.PackageKvid = price.PackageKvid;
                            reportItem.PackageUnit = price.PackageUnit;
                            reportItem.AmountPlan = price.Amount;
                        }
                        if (currentPrices.Count > 1)
                        {
                            price = currentPrices.FirstOrDefault(o => o.IsDefault);
                            if (price == null) price = currentPrices[0];
                            reportItem.PackageKvid = price.PackageKvid;
                            var packageUnitArray = new List<string>();
                            foreach (var currentPrice in currentPrices)
                            {
                                packageUnitArray.Add(currentPrice.PackageUnit);
                            }
                            reportItem.PackageUnit = string.Join(",", packageUnitArray);
                            reportItem.AmountPlan = price.Amount;
                        }

                    }
                }
                //{
                //    var conn = KiviiContext.GetOpenedDbConnection<Price>();
                //    var queryPrice = conn.From<Price>();
                //    queryPrice.Where(o => o.GoodsKvid == detection.GoodsKvid);
                //    prices = conn.Select(queryPrice).ToList();
                //}
                
                //获取价格代码
                if (reportItemOriginal.GoodsKvid != detection.GoodsKvid&&(detection.ReferenceDetectionKvid!=Guid.Empty||detection.ReferenceStandardKvid!=Guid.Empty))
                {
                    //reportItem.GoodsKvid = detection.GoodsKvid;
                    reportItem.PackageUnit = "价格关联已变更,请删除重新选择";
                }
                
                if (reportItem.StandardCode!=null&&!standards.IsNullOrEmpty())
                {
                    var standard = standards.FirstOrDefault(o =>o.Kvid==detection.StandardKvid);
                    if (standard != null) { 
                        reportItem.StandardCode = standard.Code;
                        reportItem.StandardTitle= standard.Title;
                    }
                }
            }
            //拷贝粘贴不粘贴条件值
            if (reportItem.Metadata != null && reportItem.Metadata.ContainsKey("DetectionValueCondition")) reportItem.Metadata["DetectionValueCondition"] = string.Empty;
            //拷贝粘贴不粘贴(标准值更新标记)
            if (reportItem.Metadata != null && reportItem.Metadata.ContainsKey("IsUpdateDetectionValueQualified")) reportItem.Metadata["IsUpdateDetectionValueQualified"] = string.Empty;
            //判断是否拷贝客户要求标准值
            if(!isCopyDeteValueCustomerRequired&& reportItem.Metadata != null && reportItem.Metadata.ContainsKey("DetectionValueCustomer")) reportItem.Metadata["DetectionValueCustomer"] = string.Empty;
            reportItem.TestKvid = Guid.Empty;
            reportItem.TestRootKvid = Guid.Empty;
            reportItem.TestValue = String.Empty;
            reportItem.TestCreatorKvid = Guid.Empty;
            reportItem.TestCreatorName = String.Empty;
            reportItem.TestReviewerKvid = Guid.Empty;
            reportItem.TestReviewerName = String.Empty;
            reportItem.TestCreateTime = null;
            reportItem.TestReviewTime = null;
            reportItem.TestOriginalFilePath = String.Empty;
            reportItem.Judgement = String.Empty;
            reportItem.StatusType = ReportItemStatusType.BeforeTest;
            reportItem.Status = (int)ReportItemStatusType.BeforeTest;
            return reportItem;
        }
        /// <summary>
        /// 全色号目录下
        /// 根据拷贝的检验项获取重组后的RootKvid
        /// 通过该检验项目的ParentKvid是否是原RootKvid来进行判断
        /// </summary>
        /// <param name="reportItem"></param>
        /// <param name="reportItems"></param>
        /// <param name="RootKvid"></param>
        /// <returns></returns>
        public static Guid GetRootKvid(this ReportItem reportItem, List<ReportItem> reportItems)
        {
            //最多四级目录,需要一一判定,本方法根级过滤,因此需要判断三级(色号->部位->主项->子项)
            var rootKvid = reportItem.RootKvid;
            var supReportItem = new ReportItem();
            //第一级
            if (reportItem.ParentKvid == rootKvid) return reportItem.Kvid;
            //第二级
            supReportItem = reportItems.Where(o => o.Kvid == reportItem.ParentKvid).FirstOrDefault();
            supReportItem.ThrowIfNull("未找到拷贝项上级目录!");
            if (supReportItem.ParentKvid == rootKvid) return supReportItem.Kvid;
            //第三级
            supReportItem = reportItems.Where(o => o.Kvid == supReportItem.ParentKvid).FirstOrDefault();
            supReportItem.ThrowIfNull("未找到拷贝项上级目录!");
            if (supReportItem.ParentKvid == rootKvid) return supReportItem.Kvid;
            throw new Exception("未找到粘贴项根目录!");
        }
        /// <summary>
        /// 获取与拷贝数据匹配的Kvid
        /// </summary>
        /// <param name="originalKvid"></param>
        /// <param name="MatchKvids"></param>
        /// <returns></returns>
        public static Guid GetMatchKvid(this Guid originalKvid, List<MatchKvids> MatchKvidsItems)
        {
            var replaceKvid = Guid.Empty;
            var matchKvidsItem = MatchKvidsItems.FirstOrDefault(o => o.OriginalKvid == originalKvid);
            if (matchKvidsItem == null) throw new Exception("数据匹配失败,请通知系统管理员检查数据!");
            replaceKvid = matchKvidsItem.MatchKvid;
            return replaceKvid;
        }
        /// <summary>
        /// 计算项目收费，根据合同、报价单、服务价格匹配收费
        /// </summary>
        /// <param name="reportItem"></param>
        /// <param name="report"></param>
        /// <returns></returns>
        public static ReportItem CalculateAmount(this ReportItem reportItem,Quotation quotation,List<QuotationDetail> quotationDetails,List<Price> prices)
        {
            if (reportItem.GoodsKvid == Guid.Empty) return reportItem;
            var currentPrice = new Price();
            if (reportItem.PackageUnit.IsNullOrEmpty()) return reportItem;
            var reportItemPrices = prices.Where(o => o.GoodsKvid == reportItem.GoodsKvid && o.PackageUnit == reportItem.PackageUnit.Split(',').FirstOrDefault()).ToList();
            if (reportItemPrices.Count == 0) return reportItem;
            if (reportItemPrices.Count == 1)
            {
                currentPrice = reportItemPrices[0];
            }
            else
            {
                currentPrice = reportItemPrices.FirstOrDefault(o => o.IsDefault);
                if (currentPrice == null) currentPrice = reportItemPrices[0];
            }

            //更新
            //reportItem.Amount = currentPrice.Amount;
            //reportItem.AddOnlyProperties(o => o.Amount);
            QuotationDetail quotationDetail = quotationDetails == null ? null : quotationDetails.Where(o => o.PriceKvid == currentPrice.Kvid).FirstOrDefault();

            //如果没有客户报价详细，直接用折扣
            if (quotationDetail == null)
            {
                if (quotation != null) reportItem.Amount = (decimal)(reportItem.AmountPlan * quotation.DiscountRate);
                else reportItem.Amount = reportItem.AmountPlan;
            }
            else
            {
                reportItem.Amount = quotationDetail.Amount;
            }
            reportItem.AddOnlyProperties(o => o.Amount);
            return reportItem;
        }

        /// <summary>
        /// 计算项目收费，根据合同、报价单、服务价格匹配收费
        /// </summary>
        /// <param name="reportItem"></param>
        /// <param name="report"></param>
        /// <returns></returns>
        public static ReportItem CalculateAmountUpgrade(this ReportItem reportItem, Quotation quotation, List<QuotationDetail> quotationDetails, List<Price> prices)
        {
            if (reportItem.GoodsKvid == Guid.Empty) return reportItem;
            var currentPrice = new Price();
            if (reportItem.PackageUnit.IsNullOrEmpty()) return reportItem;
            var reportItemPrices = prices.Where(o => o.GoodsKvid == reportItem.GoodsKvid && o.PackageUnit == reportItem.PackageUnit.Split(',').FirstOrDefault()).ToList();
            if (reportItemPrices.Count == 0) return reportItem;
            if (reportItemPrices.Count == 1)
            {
                currentPrice = reportItemPrices[0];
            }
            else
            {
                currentPrice = reportItemPrices.FirstOrDefault(o => o.IsDefault);
                if (currentPrice == null) currentPrice = reportItemPrices[0];
            }

            //更新
            //reportItem.Amount = currentPrice.Amount;
            //reportItem.AddOnlyProperties(o => o.Amount);
            QuotationDetail quotationDetail = quotationDetails == null ? null : quotationDetails.Where(o => o.PriceKvid == currentPrice.Kvid).FirstOrDefault();

            //如果没有客户报价详细，直接用折扣
            if (quotationDetail == null)
            {
                if (quotation != null) reportItem.Amount = (decimal)(reportItem.AmountPlan * quotation.DiscountRate);
                else reportItem.Amount = reportItem.AmountPlan;
            }
            else
            {
                reportItem.Amount = quotationDetail.Amount;
            }
            reportItem.AddOnlyProperties(o => o.Amount);
            return reportItem;
        }
        #endregion

        #region 计算检验依据等
        /// <summary>
        /// 计算报告检验依据的方法，根据参数计算报告的返回字段中是否要计算以下的结果
        /// </summary>
        /// <param name="report"></param>
        /// <param name="calcForceStandard">是否要计算强制标准项目</param>
        /// <param name="calcProductStandard">是否要计算产品标准项目</param>
        /// <param name="calcProductStandardWithoutForce">是否要计算产品标准项目，排除强制标准</param>
        /// <param name="calcMethodStandard">是否要计算方法标准</param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Report CalculateStandard(this Report report, bool combineProductAndMethod, bool calcForceStandard = false, List<string> forceStandards = null, bool calcProductStandard = false, bool calcProductStandardWithoutForce = false, bool calcMethodStandard = false,bool hasEnTitle = false, IDbConnection conn = null)
        {
            //参数calcForceStandard为true，必须传入强标list
            if (calcForceStandard && forceStandards == null) throw new Exception("参数异常，未传入强制标准列表");
            //获取报告
            (report.Kvid == Guid.Empty).ThrowIfTrue("缺少ReportKvid无法同步!");
            report.ThrowIfNull("未找到需要同步的报告!");
            bool disposeConn = conn == null;
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            var connStd = KiviiContext.GetOpenedDbConnection<Standard>();
            //获取ReportItem
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == report.Kvid).OrderBy(o => o.SortId).ThenBy(o => o.SortIdEx);

            //整个报告的全部项目
            var allReportItems = conn.Select(queryReportItems);
            if (disposeConn) conn.Dispose();
            //全项中的大项
            var primaryAllReportItems = allReportItems.filterReportItem();
            if (primaryAllReportItems.IsNullOrEmpty())
            {
                report.TestStandards = "";
                report.AddOnlyProperties(o => o.TestStandards);
                return report;
            }

            //检验依据,按需计算
            var testStandards = string.Empty;
            var testStandardsList = new List<string>();

            var testStandardsForce = string.Empty;
            var testStandardsForceList = new List<string>();

            var testStandardsProduct = string.Empty;
            var testStandardsProductList = new List<string>();

            var testStandardsProductWithoutForce = string.Empty;
            var testStandardsProductWithoutForceList = new List<string>();

            var testStandardsMethod = string.Empty;
            var testStandardsMethodList = new List<string>();

            #region 组合检验依据

            //遍历所有的大项，进行数据组合
            foreach (var item in primaryAllReportItems)
            {
                var stdEnTitle = item.GetEnStandard(connStd);
                //组合产品和方法返回
                //判断是否要加英文标准名称
                if (hasEnTitle)
                {
                    testStandardsList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》" + (stdEnTitle.IsNullOrEmpty() ? "" : "《" + stdEnTitle + "》")));
                }
                else
                {
                    testStandardsList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》"));
                }
                
                //判定强制标准
                if (forceStandards != null)
                {
                    foreach (var std in forceStandards)
                    {
                        if (item.StandardCode.Contains(std))
                        {
                            if (hasEnTitle)
                            {
                                testStandardsForceList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》" + (stdEnTitle.IsNullOrEmpty() ? "" : "《" + stdEnTitle + "》")));
                            }
                            else
                            {
                                testStandardsForceList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》"));
                            }
                        }
                    }
                }
                //方法标准
                if (item.IsMethodStandard(connStd))
                {
                    if (hasEnTitle)
                    {
                        testStandardsMethodList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》" + (stdEnTitle.IsNullOrEmpty() ? "" : "《" + stdEnTitle + "》")));
                    }
                    else
                    {
                        testStandardsMethodList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》"));
                    }
                    
                }
                //产品标准
                else
                {
                    if (hasEnTitle)
                    {
                        testStandardsProductList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》" + (stdEnTitle.IsNullOrEmpty() ? "" : "《" + stdEnTitle + "》")));
                    }
                    else
                    {
                        testStandardsProductList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》"));
                    }
                    
                    if (forceStandards != null)
                    {
                        foreach (var std in forceStandards)
                        {
                            if (!item.StandardCode.Contains(std))
                            {
                                if (hasEnTitle)
                                {
                                    testStandardsProductWithoutForceList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》" + (stdEnTitle.IsNullOrEmpty() ? "" : "《" + stdEnTitle + "》")));
                                }
                                else
                                {
                                    testStandardsProductWithoutForceList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》"));
                                }
                            }
                        }
                    }
                }
            }
            testStandardsList = testStandardsList.Distinct().ToList();
            testStandards = String.Join("、", testStandardsList);

            testStandardsProductList = testStandardsProductList.Distinct().ToList();
            testStandardsProduct = String.Join("、", testStandardsProductList);

            testStandardsProductWithoutForceList = testStandardsProductWithoutForceList.Distinct().ToList();
            testStandardsProductWithoutForce = String.Join("、", testStandardsProductWithoutForceList);

            testStandardsMethodList = testStandardsMethodList.Distinct().ToList();
            testStandardsMethod = String.Join("、", testStandardsMethodList);

            testStandardsForceList = testStandardsForceList.Distinct().ToList();
            testStandardsForce = String.Join("、", testStandardsForceList);

            //根据传入参数，组织返回数据，只有testStandardsProduct、testStandards、testStandardsProductWithoutForce使用正式字段，其他使用metadata字段
            if (combineProductAndMethod)
            {
                report.TestStandards = testStandards;
            }
            else if (calcProductStandard)
            {
                report.TestStandards = testStandardsProduct;
            }
            else if (calcProductStandardWithoutForce)
            {
                report.TestStandards = testStandardsProductWithoutForce;
            }
            else
            {
                report.TestStandards = string.Empty;
            }
            if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
            //方法标准
            if (calcMethodStandard)
            {
                report.Metadata["testStandardsMethod"] = testStandardsMethod;
                report.AddOnlyProperties(o => o.Metadata);
            }
            //国标
            if (calcForceStandard)
            {
                report.Metadata["testStandardsForce"] = testStandardsForce;
                report.AddOnlyProperties(o => o.Metadata);
            }
            report.AddOnlyProperties(o => o.TestStandards);
            #endregion
            return report;
        }
        #endregion

        #region 计算检验项目
        /// <summary>
        /// 组合检验项目方法
        /// </summary>
        /// <param name="report"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Report CalculateReportDetection(this Report report, IDbConnection conn = null)
        {
            //获取报告
            (report.Kvid == Guid.Empty).ThrowIfTrue("缺少ReportKvid无法同步检验标准及项目!");
            report.ThrowIfNull("未找到需要同步的报告!");
            bool disposeConn = conn == null;
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            //获取ReportItem
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == report.Kvid).OrderBy(o => o.SortId).ThenBy(o => o.SortIdEx);

            //整个报告的全部项目
            var allReportItems = conn.Select(queryReportItems);
            if (disposeConn) conn.Dispose();
            //全项中的大项
            var primaryAllReportItems = allReportItems.filterReportItem();
            if (primaryAllReportItems.IsNullOrEmpty())
            {
                report.TestStandards = "";
                report.AddOnlyProperties(o => o.TestStandards);
                report.TestDetections = "";
                report.AddOnlyProperties(o => o.TestDetections);
                return report;
            }
            //primaryAllReportItems.ThrowIfNullOrEmpty("此报告全部检验项目未找到检验大项,结构异常,请联系管理员!");

            //查看是否已经存在色号
            bool existSelf = allReportItems.Exists(o => o.Type == ReportItemType.SampleSelf);

            var testDetections = string.Empty;//检验项目
            var testDetectionsList = new List<string>();

            #region 组合检验项目
            //根据大项进行分组
            var groupPrimarys = primaryAllReportItems.GroupBy(o => o.Title);
            foreach (var kv in groupPrimarys)
            {
                var detectionStr = kv.Key;

                var self = new List<string>();
                var part = new List<string>();
                var actual = new List<string>();
                var method = new List<string>();

                //存在色号的情况
                if (existSelf)
                {
                    //key色号,value部位集合
                    var selfAndParts = new Dictionary<string, List<string>>();
                    foreach (var primary in kv)
                    {
                        var parentItem = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid);
                        //存在色号情况下 不可能为空  为空即跳过不处理
                        if (parentItem == null) continue;

                        var selfParent = parentItem;
                        if (parentItem.Type == ReportItemType.SamplePart) selfParent = allReportItems.FirstOrDefault(o => o.Kvid == parentItem.ParentKvid);

                        if (selfParent == null) continue;

                        var detectionMethodDescription = "";
                        if (primary.Metadata.ContainsKey("DetectionMethodDescription"))
                        {
                            detectionMethodDescription = primary.Metadata["DetectionMethodDescription"].IsNullOrEmpty() ? "" : primary.Metadata["DetectionMethodDescription"];
                        }
                        var detectionCodeActual = primary.DetectionCodeActual;
                        if (primary.Metadata.ContainsKey("DetectionCodeActual")) detectionCodeActual = primary.Metadata["DetectionCodeActual"];
                        if (!detectionMethodDescription.IsNullOrEmpty())
                        {
                            detectionCodeActual = detectionCodeActual.Replace(detectionMethodDescription, "");
                            detectionMethodDescription = " " + detectionMethodDescription;
                        }
                        if (!actual.Contains(primary.DetectionCodeActual)) actual.Add(detectionCodeActual);

                        if (!method.Contains(detectionMethodDescription)) method.Add(detectionMethodDescription);

                        //说明有部位
                        if (parentItem.Type == ReportItemType.SamplePart)
                        {
                            if (selfAndParts.ContainsKey(selfParent.Title))
                            {
                                if (!selfAndParts[selfParent.Title].Contains(parentItem.Title)) selfAndParts[selfParent.Title].Add(parentItem.Title);
                            }
                            else
                            {
                                var values = new List<string>();
                                values.Add(parentItem.Title);
                                selfAndParts.Add(selfParent.Title, values);
                            }
                        }
                        //没有部位
                        else
                        {
                            if (!selfAndParts.ContainsKey(selfParent.Title))
                            {
                                var values = new List<string>();
                                values.Add(string.Empty);
                                selfAndParts.Add(selfParent.Title, values);
                            }
                        }
                    }

                    foreach (var selfAndPart in selfAndParts)
                    {
                        self.Add(selfAndPart.Key);
                        selfAndPart.Value.RemoveAll(o => o.IsNullOrEmpty());
                        part.AddRange(selfAndPart.Value);
                    }
                }
                //没有色号的情况
                else
                {
                    foreach (var primary in kv)
                    {

                        var detectionMethodDescription = "";
                        if (primary.Metadata.ContainsKey("DetectionMethodDescription"))
                        {
                            detectionMethodDescription = primary.Metadata["DetectionMethodDescription"].IsNullOrEmpty() ? "" : primary.Metadata["DetectionMethodDescription"];
                        }
                        var detectionCodeActual = primary.DetectionCodeActual;
                        if (primary.Metadata.ContainsKey("DetectionCodeActual")) detectionCodeActual = primary.Metadata["DetectionCodeActual"];
                        if (!detectionMethodDescription.IsNullOrEmpty())
                        {
                            detectionCodeActual = detectionCodeActual.Replace(detectionMethodDescription, "");
                            detectionMethodDescription = " " + detectionMethodDescription;
                        }
                        if (!actual.Contains(primary.DetectionCodeActual)) actual.Add(detectionCodeActual);
                        if (!method.Contains(detectionMethodDescription)) method.Add(detectionMethodDescription);

                        var parentItem = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid);
                        //说明有部位
                        if (parentItem != null)
                        {
                            if (!part.Contains(parentItem.Title))
                            {
                                part.Add(parentItem.Title);
                            }
                        }
                    }
                }

                self = self.Distinct().ToList();
                part = part.Distinct().ToList();
                actual = actual.Distinct().ToList();
                method = method.Distinct().ToList();

                if (!self.IsNullOrEmpty() && !part.IsNullOrEmpty())
                {

                }

                detectionStr += (self.IsNullOrEmpty() ? string.Empty : $"[{string.Join("、", self)}]") + (part.IsNullOrEmpty() ? string.Empty : $"({string.Join("、", part)})") + (actual.IsNullOrEmpty() ? (method.IsNullOrEmpty() ? string.Empty : "(" + string.Join("、", method) + ")") : "(" + string.Join("、", actual) + string.Join("、", method) + ")");
                testDetectionsList.Add(detectionStr);
            }
            testDetections = String.Join("、", testDetectionsList);
            report.TestDetections = testDetections;
            report.AddOnlyProperties(o => o.TestDetections);
            #endregion
            return report;
        }
        #endregion

        #region 用于判断ReportItem用的标准，是否为方法标准
        private static bool IsMethodStandard(this ReportItem reportItem, IDbConnection stdConn = null)
        {
            if(reportItem.StandardKvid!=null)
            {
                if (stdConn == null) stdConn = KiviiContext.GetOpenedDbConnection<Standard>();
                var stdandard = stdConn.SingleById<Standard>(reportItem.StandardKvid);
                if (stdandard != null)
                {
                    if (stdandard.Type == "Original") return true;
                }
            }
            return false;
        }
        #endregion

        #region 用于查询ReportItem用的标准有无英文名称
        private static string GetEnStandard(this ReportItem reportItem, IDbConnection stdConn = null)
        {
            if (reportItem.StandardKvid != null)
            {
                if (stdConn == null) stdConn = KiviiContext.GetOpenedDbConnection<Standard>();
                var stdandard = stdConn.SingleById<Standard>(reportItem.StandardKvid);
                if (stdandard != null)
                {
                    if (!stdandard.TitleEnglish.IsNullOrEmpty()) return stdandard.TitleEnglish;
                }
            }
            return "";
        }
        #endregion
    }
}
