﻿//using Jst.Lims.Entities;
using Kivii.Lac.Lims.RabbitMq.Entities;
using Kivii;
using Kivii.Lims.Entities;
using Kivii.Lac.Lims.Extensions;
using Kivii.Lac.Lims.Transforms;
using Kivii.Linq;
using Kivii.Messaging;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Kivii.Domain.Lac.Lims.Features
{
    public class DomainFeature : IPlugin, IPreInitPlugin
    {
        public void Configure(IAppHost appHost)
        {
            appHost.RegisterTypedResponseFilter<ReportAssignResponse>(AssignIssue);
            appHost.RegisterTypedResponseFilter<ReportGenerateResponse>(GenerateFile);
        }


        /// <summary>
        /// 监听报告是否已经到报告归档状态,同时要去更新老Lims对应报告 如果此份报告存在的话
        /// </summary>
        /// <param name="req"></param>
        /// <param name="resp"></param>
        /// <param name="rtns"></param>
        private void AssignIssue(IRequest req, IResponse resp, ReportAssignResponse rtns)
        {
            if (rtns == null) return;
            if (rtns.Results.IsNullOrEmpty()) return;
            IMessageProducer _messageProducer = null;
            if (_messageProducer == null) _messageProducer = (req.TryResolve<IMessageFactory>() ?? req.TryResolve<IMessageService>()?.MessageFactory)?.CreateMessageProducer();

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            #region 筛选到报告编辑的报告 进行绩效考核
            var reportKvids = rtns.Results.Where(o => o.Status == (int)ReportStatusType.ReportPreparation).ToList().ConvertAll(o => o.Kvid);
            List<ReportItem> reportItems = new List<ReportItem>();
            List<ReportItem> reportPartItems = new List<ReportItem>();
            List<ReportItem> reportSelfItems = new List<ReportItem>();
            List<ReportItem> reportMainItems = new List<ReportItem>();
            if (!reportKvids.IsNullOrEmpty()) reportItems = conn.Select<ReportItem>(o => Sql.In(o.ReportKvid, reportKvids));
            if (!reportItems.IsNullOrEmpty()) reportMainItems = reportItems.filterReportItem();
            if (!reportItems.IsNullOrEmpty()) reportPartItems = reportItems.Where(o => o.Type == ReportItemType.SamplePart).ToList();
            if (!reportItems.IsNullOrEmpty()) reportSelfItems = reportItems.Where(o => o.Type == ReportItemType.SampleSelf).ToList();

            foreach (var self in reportSelfItems)
            {
                var mains = reportMainItems.Where(o => o.RootKvid == self.Kvid).ToList();
                foreach (var main in mains)
                {
                    main.TitleSelf = self.Title;
                    //main.TitlePart = $"[{self.Title}]";
                }
            }
            foreach (var part in reportPartItems)
            {
                var mains = reportMainItems.Where(o => o.ParentKvid == part.Kvid).ToList();
                foreach (var main in mains)
                {
                    main.TitlePart = part.Title;
                    //main.TitlePart += $"[{part.Title}]";
                }
            }
            #endregion

            foreach (var item in rtns.Results)
            {
                //报告签发至 打印或者归档时操作合成pdf及生成二维码跳转地址
                if (item.Status == (int)ReportStatusType.ReportPrint || item.Status == (int)ReportStatusType.ReportCollected)
                {
                    #region 合成pdf信息推送至消息队列
                    try
                    {
                        //要是为空 说明平台没有启用RabbitMq服务 没有生产端 就不推了
                        if (_messageProducer == null) return;
                        var request = new ReportMqRequest();
                        request.PopulateWith(item);
                        if (request.Metadata.IsNullOrEmpty())
                        {
                            request.Metadata = new Dictionary<string, string>();
                            request.Metadata = item.Metadata;
                        }
                        foreach (var prop in item.GetType().GetProperties())
                        {
                            try
                            {
                                var key = prop.Name;
                                if (key == "Metadata") continue;
                                var type = prop.GetType();
                                var value = prop.GetValue(item, null);
                                if (value == null) continue;
                                request.Metadata[key] = value.ToString();
                                if (type == typeof(DateTime)) request.Metadata[key] = DateTime.Parse(value.ToString()).ToString("yyyy-MM-dd");
                            }
                            catch { continue; }
                        }
                        //request.Kvid = item.Kvid;
                        //request.ReportId = item.ReportId;

                        request.SaveFilePath = $"/Storages/Reports/Pdf/{item.Kvid}/{item.ReportId}.pdf";
                        request.OpenFilePath = $"/Storages/Report/Excel01/{item.Kvid}/{item.ReportId}.xlsx";
                        if (!item.TemplateName.IsNullOrEmpty())
                        {
                            request.Stamp = item.TemplateName;
                            if (item.TemplateName.Contains("英文")&& !item.TemplateName.Contains("中英文")) request.Stamp = "新英文印章";
                        }
                        _messageProducer.Publish(request);
                    }
                    catch (Exception ex)
                    {
                        var log = new EntityLog<Report>();
                        log.OwnerKvid = item.Kvid;
                        log.BizId = item.ReportId;
                        log.BizKvid = item.Kvid;
                        log.BizType = typeof(Report).FullName;
                        log.Type = "生成PDF";
                        log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                        log.Remark = $"报告编号:{item.ReportId} 推送生成PDF队列失败,原因:{ex.Message}";
                        conn.Insert(log);
                    }
                    #endregion

                }
                
            }

            _messageProducer?.Dispose();
            conn.Dispose();
            //connOldLims.Dispose();
        }

        /// <summary>
        /// 监听报告是否重新合并生成
        /// 重新合并生成后要执行 推送消息 处理Excel转PDF操作
        /// </summary>
        /// <param name="req"></param>
        /// <param name="resp"></param>
        /// <param name="rtns"></param>
        private void GenerateFile(IRequest req, IResponse resp, ReportGenerateResponse rtns)
        {
            if (rtns == null) return;
            //不是合并生成Excel
            if (!rtns.IsCombine) return;
            if (rtns.Results.IsNullOrEmpty()) return;
            IMessageProducer _messageProducer = null;
            if (_messageProducer == null) _messageProducer = (req.TryResolve<IMessageFactory>() ?? req.TryResolve<IMessageService>()?.MessageFactory)?.CreateMessageProducer();
            //要是为空 说明平台没有启用RabbitMq服务 没有生产端 就不推了
            if (_messageProducer == null) return;

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            
            foreach (var item in rtns.Results)
            {
                try
                {
                    var request = new ReportMqRequest();
                    request.PopulateWith(item);
                    if (request.Metadata.IsNullOrEmpty())
                    {
                        request.Metadata = new Dictionary<string, string>();
                        request.Metadata = item.Metadata;
                    }
                    foreach (var prop in item.GetType().GetProperties())
                    {
                        try
                        {
                            var key = prop.Name;
                            if (key == "Metadata") continue;
                            var type = prop.GetType();
                            var value = prop.GetValue(item, null);
                            if (value == null) continue;
                            request.Metadata[key] = value.ToString();
                            if (type == typeof(DateTime)) request.Metadata[key] = DateTime.Parse(value.ToString()).ToString("yyyy-MM-dd");
                        }
                        catch { continue; }
                    }
                    request.SaveFilePath = $"/Storages/Reports/Pdf/{item.Kvid}/{item.ReportId}.pdf";
                    request.OpenFilePath = $"/Storages/Report/Excel01/{item.Kvid}/{item.ReportId}.xlsx";
                    if (!item.TemplateName.IsNullOrEmpty())
                    {
                        request.Stamp = item.TemplateName;
                        if (item.TemplateName.Contains("英文")) request.Stamp = "新英文印章";
                    }
                    _messageProducer.Publish(request);

                }
                catch(Exception ex)
                {
                    var log = new EntityLog<Report>();
                    log.OwnerKvid = item.Kvid;
                    log.BizId = item.ReportId;
                    log.BizKvid = item.Kvid;
                    log.BizType = typeof(Report).FullName;
                    log.Type = "生成PDF";
                    log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    log.Remark = $"报告编号:{item.ReportId} 推送生成PDF队列失败,原因:{ex.Message}";
                    conn.Insert(log);
                }
            }
            _messageProducer?.Dispose();
        }

        public void Register(IAppHost appHost)
        {
        }

        private List<string> EmailSplit(string address)
        {
            if (address.IsNullOrEmpty()) return null;
            var addresses = address.Split(',', '，', ';', '；').ToList();
            var rtns = new List<string>();
            foreach (var item in addresses)
            {
                try
                {
                    var addr = new System.Net.Mail.MailAddress(item);
                    rtns.Add(item);
                }
                catch { continue; }
            }
            return rtns;
        }
    }
}
