﻿using Kivii.Domain.Lac.Lims.Extensions;
using Kivii;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Products.Entities;
using Kivii.Standards.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Domain.Lac.Lims.Transforms
{
    #region 项目模板选择检验项目同步价格、同步检验组
    /// <summary>
    /// 通过浏览标准项目生成检验项目
    /// 同步设置生成项目的价格、检验组信息
    /// </summary>
    [Api(Description = "项目选择")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class TemplateDetailDetection : RestfulExecution<TemplateDetail>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> DetectionKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            OwnerKvid.ThrowIfEmpty("未传入TemplateKvid！");
            DetectionKvids.ThrowIfNullOrEmpty("未传入检测项目或检测项目为空！");
            #endregion

            var connLims = KiviiContext.GetOpenedDbConnection<Template>();

            //判断是否要添加的项目在当前模板下已经存在了
            var queryTemplateDetailDetectionKvids = connLims.From<TemplateDetail>();
            queryTemplateDetailDetectionKvids.Where(o => o.OwnerKvid == OwnerKvid);
            queryTemplateDetailDetectionKvids.Select(o => o.DetectionKvid);
            var templateDetailDetectionKvids = connLims.Select<Guid>(queryTemplateDetailDetectionKvids);

            templateDetailDetectionKvids.ForEach(o => DetectionKvids.Remove(o));
            if (DetectionKvids.IsNullOrEmpty()) throw new Exception("所选均为重复项,已忽略！");

            #region 获取Detections和ReferenceDetections
            var connStandards = KiviiContext.GetOpenedDbConnection<Standard>();
            var detections = connStandards.Select<Detection>(o => Sql.In(o.Kvid, DetectionKvids));
            detections.ThrowIfNullOrEmpty("未找到标准检验项目!");
            detections.Exists(o => o.StandardKvid == Guid.Empty).ThrowIfTrue("存在无标准的检验项目!");

            var standards = connStandards.Select<Standard>(o => Sql.In(o.Kvid, detections.Select(p => p.StandardKvid))).OrderBy(o => o.Code).ToList();
            standards.IsNullOrEmpty().ThrowIfTrue("未找到对应标准!");

            List<Detection> referenceDetections = null;//选择的项目的引用项目

            var refenceDetectionKvids = detections.Where(o => o.ReferenceDetectionKvid != Guid.Empty).Select(o => o.ReferenceDetectionKvid).ToList();
            refenceDetectionKvids = refenceDetectionKvids.Distinct().ToList();
            if (!refenceDetectionKvids.IsNullOrEmpty()) referenceDetections = connStandards.Select<Detection>(o => Sql.In(o.Kvid, refenceDetectionKvids));
            #endregion

            #region 获取当前树形结构同级别的StandardSortId的最大值
            var standartSortId = 0;
            var queryTemplateDetailRoot = connLims.From<TemplateDetail>();
            queryTemplateDetailRoot.Where(o => o.ParentKvid == Guid.Empty && o.OwnerKvid == OwnerKvid);
            queryTemplateDetailRoot.Select(o => o.SortId);
            var TemplateDetailStandardSortIds = connLims.Select(queryTemplateDetailRoot);
            if (!TemplateDetailStandardSortIds.IsNullOrEmpty())
            {
                foreach (var en in TemplateDetailStandardSortIds)
                {
                    standartSortId = en.SortId > standartSortId ? en.SortId : standartSortId;
                }
            }
            #endregion

            #region 获取报价信息
            var connProduct = KiviiContext.GetOpenedDbConnection<Quotation>();
            var goodsKvids = detections.Where(o => o.GoodsKvid != Guid.Empty).Select(o => o.GoodsKvid).ToList();
            if (!referenceDetections.IsNullOrEmpty()) goodsKvids.AddRange(referenceDetections.Where(o => o.GoodsKvid != Guid.Empty).Select(o => o.GoodsKvid).ToList());
            goodsKvids = goodsKvids.Distinct().ToList();

            List<Price> prices = null;
            if (!goodsKvids.IsNullOrEmpty()) prices = connProduct.Select<Price>(o => Sql.In(o.GoodsKvid, goodsKvids));
            #endregion

            var rtns = new RestfulQueryResponse<TemplateDetail>();
            rtns.Results = new List<TemplateDetail>();
            //获取涉及的标准值数据
            var queryDetectionValues = connStandards.From<DetectionValue>();
            queryDetectionValues.Where(o => Sql.In(o.DetectionKvid, DetectionKvids));
            var detectionValues = connStandards.Select(queryDetectionValues);

            IDbTransaction trans = null;//define a null tag
             trans = connLims.OpenTransaction();
            try
            {
                var groups = detections.GroupBy(o => o.StandardKvid);
                foreach (var kv in groups)
                {
                    standartSortId++;
                    foreach (var detection in kv)
                    {
                        if (detection.ParentKvid != Guid.Empty) continue;
                        Standard standard = null;
                        //处理子项Detection
                        var nodeDetections = kv.Where(o => o.ParentKvid == detection.Kvid).ToList();
                        standard = standards.FirstOrDefault(o => o.Kvid == detection.StandardKvid);
                        (standard == null).ThrowIfTrue($"未找到当前检验项[{detection.Title}]的标准!");
                        var templateDetail = new TemplateDetail();
                        templateDetail.Kvid = Guid.NewGuid();
                        templateDetail.ParentKvid = Guid.Empty;
                        templateDetail.RootKvid = templateDetail.Kvid;
                        templateDetail.OwnerKvid = OwnerKvid;
                        templateDetail.Type = nodeDetections.IsNullOrEmpty() ? TemplateDetailType.Test : TemplateDetailType.TestFolder;
                        templateDetail.StandardKvid = standard.Kvid;
                        templateDetail.StandardCode = standard.Code;
                        templateDetail.StandardTitle = standard.Title;
                        templateDetail.DetectionKvid = detection.Kvid;
                        templateDetail.DetectionCodeActual = detection.CodeActual;
                        templateDetail.DetectionTitle = detection.Title;
                        templateDetail.DetectionReferenceKvid = detection.ReferenceDetectionKvid;
                        templateDetail.Title = detection.Title;
                        templateDetail.SortIdEx = detection.SortId;
                        templateDetail.SortId = standartSortId;
                        templateDetail.Currency = CurrencyUnit.CNY;
                        templateDetail.DetectionUnit = detection.Unit;
                        templateDetail.DetectionValueQualified = detection.GetQualifiedValue(null, null, detectionValues);
                        templateDetail.Metadata = new Dictionary<string, string>();
                        templateDetail.Metadata.Add("IsMultiReference", detection.IsMultiReference.ToString());
                        templateDetail.Metadata.Add("DetectionCodeActual", detection.CodeActual);
                        templateDetail.Metadata.Add("DetectionMethodDescription", detection.MethodDescription);

                        #region  大项设置GoodsKvid
                        if (detection.GoodsKvid != Guid.Empty)
                        {
                            templateDetail.GoodsKvid = detection.GoodsKvid;
                        }
                        else if (detection.ReferenceDetectionKvid != Guid.Empty)//是引用项目，找原生项目集合
                        {
                            if (!referenceDetections.IsNullOrEmpty())
                            {
                                var refenceDetection = referenceDetections.FirstOrDefault(o => o.Kvid == detection.ReferenceDetectionKvid);
                                if (refenceDetection != null)
                                {
                                    templateDetail.GoodsKvid = refenceDetection.GoodsKvid;
                                }
                            }
                        }
                        #endregion

                        #region 设置价格相关
                        if (templateDetail.GoodsKvid != Guid.Empty && !prices.IsNullOrEmpty())
                        {
                            var testPrices = prices.Where(o => o.GoodsKvid == templateDetail.GoodsKvid).ToList();
                            if (!testPrices.IsNullOrEmpty())
                            {
                                Price testPrice = null;
                                if (testPrices.Count == 1)
                                {
                                    testPrice = testPrices[0];
                                    templateDetail.PackageKvid = testPrice.PackageKvid;
                                    templateDetail.PackageUnit = testPrice.PackageUnit;
                                }
                                else if (testPrices.Count > 1)
                                {
                                    //若有默认，PackageKvid赋默认值
                                    testPrice = testPrices.FirstOrDefault(o => o.IsDefault);
                                    if (testPrice == null)
                                    {
                                        testPrice = testPrices[0];
                                    }
                                    templateDetail.PackageKvid = testPrice.PackageKvid;
                                    testPrices = testPrices.OrderByDescending(o => o.IsDefault).ToList();
                                    var packageUnitArray = new List<string>();
                                    foreach (var en in testPrices)
                                    {
                                        packageUnitArray.Add(en.PackageUnit);
                                    }
                                    templateDetail.PackageUnit = string.Join(",", packageUnitArray);
                                }
                                templateDetail.Amount = testPrice.Amount;
                            }
                        }
                        #endregion

                        #region 设置检验组信息
                        var queryTaskInfo = connStandards.From<EntityRelation<Detection>>();
                        queryTaskInfo.Where(o => o.EntityKvid == detection.Kvid && o.RelationType == "Kivii.Organizations.Entities.Organization");
                        var taskInfo = connStandards.Select(queryTaskInfo).FirstOrDefault();
                        if (taskInfo == null)
                        {
                            templateDetail.WorkGroupName = "暂未关联检验组";
                            templateDetail.WorkGroupKvid = Guid.Empty;
                        }
                        else
                        {
                            templateDetail.WorkGroupName = taskInfo.InternalCode;
                            templateDetail.WorkGroupKvid = taskInfo.RelationKvid;
                        }
#endregion

                        connLims.Insert(templateDetail);
                        templateDetail.RemoveAllOnlyProperties();
                        rtns.Results.Add(templateDetail);

                        foreach (var node in nodeDetections)
                        {
                            var nodeTemplateDetail = new TemplateDetail();
                            nodeTemplateDetail.RootKvid = templateDetail.RootKvid;
                            nodeTemplateDetail.ParentKvid = templateDetail.Kvid;
                            nodeTemplateDetail.OwnerKvid = templateDetail.OwnerKvid;
                            nodeTemplateDetail.Type = TemplateDetailType.Test;
                            nodeTemplateDetail.StandardKvid = detection.StandardKvid;
                            nodeTemplateDetail.DetectionKvid = node.Kvid;
                            nodeTemplateDetail.DetectionReferenceKvid = node.ReferenceDetectionKvid;
                            nodeTemplateDetail.WorkGroupName = templateDetail.WorkGroupName;
                            nodeTemplateDetail.WorkGroupKvid = templateDetail.WorkGroupKvid;
                            nodeTemplateDetail.Title = node.Title;
                            nodeTemplateDetail.StandardCode = node.Code;
                            nodeTemplateDetail.Currency = CurrencyUnit.CNY;
                            nodeTemplateDetail.SortIdEx = node.SortId;
                            nodeTemplateDetail.SortId = standartSortId;
                            nodeTemplateDetail.DetectionUnit = node.Unit;
                            nodeTemplateDetail.DetectionValueQualified = node.GetQualifiedValue(null, null, detectionValues);
                            nodeTemplateDetail.Metadata = new Dictionary<string, string>();
                            nodeTemplateDetail.Metadata.Add("IsMultiReference", node.IsMultiReference.ToString());
                            connLims.Insert(nodeTemplateDetail);
                            nodeTemplateDetail.RemoveAllOnlyProperties();
                            rtns.Results.Add(nodeTemplateDetail);
                        }
                    }
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }

    }
#endregion
}
